@@ -95,6 +95,19 @@ class QtAudioDeviceManager
         return getAudioDevices(AudioManager.GET_DEVICES_INPUTS);
     }
 
+    @UsedFromNativeCode
+    static AudioDeviceInfo getAudioInputDeviceInfo(int id)
+    {
+        final AudioDeviceInfo[] audioDevices = m_audioManager.getDevices(AudioManager.GET_DEVICES_INPUTS);
+
+        for (AudioDeviceInfo deviceInfo : audioDevices) {
+            if (deviceInfo.getId() == id)
+                return deviceInfo;
+        }
+
+        return null;
+    }
+
     @UsedFromNativeCode
     static int getDefaultSampleRate()
     {
@@ -114,9 +127,6 @@ class QtAudioDeviceManager
         }
     }
 
-    // FIXME: It's quite misleading to "set" audio input/output when no MediaRecorder is used. We
-    // should more likely be "preparing" Android for the use of a specific audio device in a
-    // desired mode
     @UsedFromNativeCode
     static boolean prepareAudioInput(int id)
     {
