@@ -12,6 +12,7 @@
 
 #include <QtCore/qmetaobject.h>
 #include <QtCore/qatomic.h>
+#include <QtCore/private/qcore_mac_p.h>
 
 QT_USE_NAMESPACE
 
@@ -48,7 +49,8 @@ using AVFAtomicInt64 = QAtomicInteger<qint64>;
 - (bool)addWriterInputs;
 - (void)setQueues;
 - (void)updateDuration:(CMTime)newTimeStamp;
-- (CMSampleBufferRef)adjustTime:(CMSampleBufferRef)sample by:(CMTime)offset;
+- (QCFType<CMSampleBufferRef>)adjustTime:(const QCFType<CMSampleBufferRef> &)sample
+                                      by:(CMTime)offset;
 @end
 
 @implementation QT_MANGLE_NAMESPACE(AVFMediaAssetWriter)
@@ -97,16 +99,6 @@ using AVFAtomicInt64 = QAtomicInteger<qint64>;
         m_delegate = delegate;
         m_setStartTime = true;
         m_state.storeRelaxed(WriterStateIdle);
-        m_startTime = kCMTimeInvalid;
-        m_lastTimeStamp = kCMTimeInvalid;
-        m_lastAudioTimestamp = kCMTimeInvalid;
-        m_lastVideoTimestamp = kCMTimeInvalid;
-        m_timeOffset = kCMTimeInvalid;
-        m_adjustTime = false;
-        m_durationInMs.storeRelaxed(0);
-        m_audioSettings = nil;
-        m_videoSettings = nil;
-        m_writeFirstAudioBuffer = false;
     }
 
     return self;
@@ -309,7 +301,8 @@ using AVFAtomicInt64 = QAtomicInteger<qint64>;
     m_setStartTime = false;
 }
 
-- (CMSampleBufferRef)adjustTime:(CMSampleBufferRef)sample by:(CMTime)offset
+- (QCFType<CMSampleBufferRef>)adjustTime:(const QCFType<CMSampleBufferRef> &)sample
+                                      by:(CMTime)offset
 {
     CMItemCount count;
     CMSampleBufferGetSampleTimingInfoArray(sample, 0, nil, &count);
@@ -359,8 +352,8 @@ using AVFAtomicInt64 = QAtomicInteger<qint64>;
 }
 
 - (void)captureOutput:(AVCaptureOutput *)captureOutput
-        didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer
-        fromConnection:(AVCaptureConnection *)connection
+        didOutputSampleBuffer:(CMSampleBufferRef)buffer
+               fromConnection:(AVCaptureConnection *)connection
 {
     Q_UNUSED(connection);
     Q_ASSERT(m_service && m_service->session());
@@ -381,12 +374,13 @@ using AVFAtomicInt64 = QAtomicInteger<qint64>;
         return;
     }
 
-    if (!CMSampleBufferDataIsReady(sampleBuffer)) {
+    if (!CMSampleBufferDataIsReady(buffer)) {
         qWarning() << Q_FUNC_INFO << "sample buffer is not ready, skipping.";
         return;
     }
 
-    CFRetain(sampleBuffer);
+    // take ownership
+    auto sampleBuffer = QCFType<CMSampleBufferRef>::constructFromGet(buffer);
 
     bool isVideoBuffer = true;
     isVideoBuffer = (captureOutput != m_service->session()->audioOutput());
@@ -414,10 +408,8 @@ using AVFAtomicInt64 = QAtomicInteger<qint64>;
         }
     }
 
-    if (m_state.loadAcquire() != WriterStateActive) {
-        CFRelease(sampleBuffer);
+    if (m_state.loadAcquire() != WriterStateActive)
         return;
-    }
 
     if (m_adjustTime) {
         CMTime currentTimestamp = CMSampleBufferGetPresentationTimeStamp(sampleBuffer);
@@ -439,7 +431,6 @@ using AVFAtomicInt64 = QAtomicInteger<qint64>;
     }
 
     if (m_timeOffset.value > 0) {
-        CFRelease(sampleBuffer);
         sampleBuffer = [self adjustTime:sampleBuffer by:m_timeOffset];
     }
 
@@ -454,13 +445,11 @@ using AVFAtomicInt64 = QAtomicInteger<qint64>;
         dispatch_async(m_writerQueue, ^{
             [self writeVideoSampleBuffer:sampleBuffer];
             m_writeFirstAudioBuffer = true;
-            CFRelease(sampleBuffer);
         });
     } else if (m_writeFirstAudioBuffer) {
         m_lastAudioTimestamp = currentTimestamp;
         dispatch_async(m_writerQueue, ^{
             [self writeAudioSampleBuffer:sampleBuffer];
-            CFRelease(sampleBuffer);
         });
     }
 }
@@ -476,14 +465,30 @@ using AVFAtomicInt64 = QAtomicInteger<qint64>;
     if (m_videoQueue)
     {
         Q_ASSERT(session->videoCaptureDevice() && session->videoOutput() && session->videoOutput()->videoDataOutput());
-        m_cameraWriterInput.reset([[AVAssetWriterInput alloc] initWithMediaType:AVMediaTypeVideo
-                                                          outputSettings:m_videoSettings
-                                                          sourceFormatHint:session->videoCaptureDevice().activeFormat.formatDescription]);
+        @try {
+            m_cameraWriterInput.reset([[AVAssetWriterInput alloc]
+                    initWithMediaType:AVMediaTypeVideo
+                       outputSettings:m_videoSettings
+                     sourceFormatHint:session->videoCaptureDevice()
+                                              .activeFormat.formatDescription]);
+        } @catch (NSException *exception) {
+            qCWarning(qLcCamera) << Q_FUNC_INFO << "Failed to create video writer input:"
+                                 << QString::fromNSString(exception.reason);
+            m_cameraWriterInput.reset();
+            return false;
+        }
 
-        if (m_cameraWriterInput && [m_assetWriter canAddInput:m_cameraWriterInput]) {
-            [m_assetWriter addInput:m_cameraWriterInput];
-        } else {
-            qCDebug(qLcCamera) << Q_FUNC_INFO << "failed to add camera writer input";
+        @try {
+            if (m_cameraWriterInput && [m_assetWriter canAddInput:m_cameraWriterInput]) {
+                [m_assetWriter addInput:m_cameraWriterInput];
+            } else {
+                qCDebug(qLcCamera) << Q_FUNC_INFO << "failed to add camera writer input";
+                m_cameraWriterInput.reset();
+                return false;
+            }
+        } @catch (NSException *exception) {
+            qCWarning(qLcCamera) << Q_FUNC_INFO << "Failed to add video input:"
+                                 << QString::fromNSString(exception.reason);
             m_cameraWriterInput.reset();
             return false;
         }
@@ -493,22 +498,44 @@ using AVFAtomicInt64 = QAtomicInteger<qint64>;
 
     m_audioWriterInput.reset();
     if (m_audioQueue) {
-        m_audioWriterInput.reset([[AVAssetWriterInput alloc] initWithMediaType:AVMediaTypeAudio
-                                                             outputSettings:m_audioSettings]);
+        @try {
+            m_audioWriterInput.reset([[AVAssetWriterInput alloc]
+                    initWithMediaType:AVMediaTypeAudio
+                       outputSettings:m_audioSettings]);
+        } @catch (NSException *exception) {
+            qCWarning(qLcCamera) << Q_FUNC_INFO << "Failed to create audio writer input:"
+                                 << QString::fromNSString(exception.reason);
+            m_audioWriterInput.reset();
+            // But we still can record video.
+            if (!m_cameraWriterInput)
+                return false;
+        }
         if (!m_audioWriterInput) {
             qWarning() << Q_FUNC_INFO << "failed to create audio writer input";
             // But we still can record video.
             if (!m_cameraWriterInput)
                 return false;
-        } else if ([m_assetWriter canAddInput:m_audioWriterInput]) {
-            [m_assetWriter addInput:m_audioWriterInput];
-            m_audioWriterInput.data().expectsMediaDataInRealTime = YES;
         } else {
-            qWarning() << Q_FUNC_INFO << "failed to add audio writer input";
-            m_audioWriterInput.reset();
-            if (!m_cameraWriterInput)
-                return false;
-            // We can (still) write video though ...
+            @try {
+                if ([m_assetWriter canAddInput:m_audioWriterInput]) {
+                    [m_assetWriter addInput:m_audioWriterInput];
+                    m_audioWriterInput.data().expectsMediaDataInRealTime = YES;
+                } else {
+                    qWarning() << Q_FUNC_INFO << "failed to add audio writer input";
+                    m_audioWriterInput.reset();
+                    if (!m_cameraWriterInput)
+                        return false;
+                    // We can (still) write video though ...
+                }
+            } @catch (NSException *exception) {
+                qCWarning(qLcCamera)
+                        << Q_FUNC_INFO
+                        << "Failed to add audio input:" << QString::fromNSString(exception.reason);
+                m_audioWriterInput.reset();
+                if (!m_cameraWriterInput)
+                    return false;
+                // We can (still) write video though ...
+            }
         }
     }
 
