@@ -1,9 +1,12 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
-#include <common/qglist_helper_p.h>
 #include "qgstreamerformatinfo_p.h"
 
+#include <common/qglist_helper_p.h>
+
+#include <QtCore/qset.h>
+
 #include <gst/gst.h>
 
 QT_BEGIN_NAMESPACE
@@ -79,10 +82,8 @@ QMediaFormat::VideoCodec QGstreamerFormatInfo::videoCodecForCaps(QGstStructureVi
     if (name == "x-h264"sv)
         return QMediaFormat::VideoCodec::H264;
 
-#if GST_CHECK_VERSION(1, 17, 0) // x265enc seems to be broken on 1.16 at least
     if (name == "x-h265"sv)
         return QMediaFormat::VideoCodec::H265;
-#endif
 
     if (name == "x-vp8"sv)
         return QMediaFormat::VideoCodec::VP8;
@@ -180,7 +181,8 @@ QImageCapture::FileFormat QGstreamerFormatInfo::imageFormatForCaps(QGstStructure
     return QImageCapture::UnspecifiedFormat;
 }
 
-static QPair<QList<QMediaFormat::AudioCodec>, QList<QMediaFormat::VideoCodec>> getCodecsList(bool decode)
+static std::pair<QList<QMediaFormat::AudioCodec>, QList<QMediaFormat::VideoCodec>>
+getCodecsList(bool decode)
 {
     QList<QMediaFormat::AudioCodec> audio;
     QList<QMediaFormat::VideoCodec> video;
@@ -193,7 +195,7 @@ static QPair<QList<QMediaFormat::AudioCodec>, QList<QMediaFormat::VideoCodec>> g
     for (GstElementFactory *factory :
          QGstUtils::GListRangeAdaptor<GstElementFactory *>(elementList)) {
         for (GstStaticPadTemplate *padTemplate :
-             QGstUtils::GListRangeAdaptor<GstStaticPadTemplate *>(
+             QGstUtils::GListConstRangeAdaptor<GstStaticPadTemplate *>(
                      gst_element_factory_get_static_pad_templates(factory))) {
             if (padTemplate->direction == padDirection) {
                 auto caps = QGstCaps(gst_static_caps_get(&padTemplate->static_caps), QGstCaps::HasRef);
@@ -237,7 +239,7 @@ QGstreamerFormatInfo::getCodecMaps(QMediaFormat::ConversionMode conversionMode,
         QList<QMediaFormat::FileFormat> fileFormats;
 
         for (GstStaticPadTemplate *padTemplate :
-             QGstUtils::GListRangeAdaptor<GstStaticPadTemplate *>(
+             QGstUtils::GListConstRangeAdaptor<GstStaticPadTemplate *>(
                      gst_element_factory_get_static_pad_templates(factory))) {
 
             // Check pads on data side for file formats, except for parsers check source side
@@ -262,7 +264,7 @@ QGstreamerFormatInfo::getCodecMaps(QMediaFormat::ConversionMode conversionMode,
         QList<QMediaFormat::VideoCodec> videoCodecs;
 
         for (GstStaticPadTemplate *padTemplate :
-             QGstUtils::GListRangeAdaptor<GstStaticPadTemplate *>(
+             QGstUtils::GListConstRangeAdaptor<GstStaticPadTemplate *>(
                      gst_element_factory_get_static_pad_templates(factory))) {
 
             // check the other side for supported inputs/outputs
@@ -327,7 +329,7 @@ static QList<QImageCapture::FileFormat> getImageFormatList()
          QGstUtils::GListRangeAdaptor<GstElementFactory *>(elementList)) {
 
         for (GstStaticPadTemplate *padTemplate :
-             QGstUtils::GListRangeAdaptor<GstStaticPadTemplate *>(
+             QGstUtils::GListConstRangeAdaptor<GstStaticPadTemplate *>(
                      gst_element_factory_get_static_pad_templates(factory))) {
             if (padTemplate->direction == GST_PAD_SRC) {
                 QGstCaps caps = QGstCaps(gst_static_caps_get(&padTemplate->static_caps), QGstCaps::HasRef);
