@@ -2,10 +2,12 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <system_error>
+#include <mfapi.h>
 #include <mferror.h>
 #include <qlogging.h>
 #include <qdebug.h>
 #include "mfdecodersourcereader_p.h"
+#include "mfdecodersourcereadercallback_p.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -14,12 +16,25 @@ ComPtr<IMFMediaType> MFDecoderSourceReader::setSource(IMFMediaSource *source, QA
     ComPtr<IMFMediaType> mediaType;
     m_sourceReader.Reset();
 
+    if (m_sourceReaderCallback) {
+        disconnect(m_sourceReaderCallback.Get());
+
+        m_sourceReaderCallback.Reset();
+    }
+
     if (!source)
         return mediaType;
 
+    m_sourceReaderCallback = makeComObject<MFSourceReaderCallback>();
+
+    connect(m_sourceReaderCallback.Get(), &MFSourceReaderCallback::finished, this,
+            &MFDecoderSourceReader::finished);
+    connect(m_sourceReaderCallback.Get(), &MFSourceReaderCallback::newSample, this,
+            &MFDecoderSourceReader::newSample);
+
     ComPtr<IMFAttributes> attr;
     MFCreateAttributes(attr.GetAddressOf(), 1);
-    if (FAILED(attr->SetUnknown(MF_SOURCE_READER_ASYNC_CALLBACK, this)))
+    if (FAILED(attr->SetUnknown(MF_SOURCE_READER_ASYNC_CALLBACK, m_sourceReaderCallback.Get())))
         return mediaType;
     if (FAILED(attr->SetUINT32(MF_SOURCE_READER_DISCONNECT_MEDIASOURCE_ON_SHUTDOWN, TRUE)))
         return mediaType;
@@ -52,52 +67,6 @@ void MFDecoderSourceReader::readNextSample()
         m_sourceReader->ReadSample(MF_SOURCE_READER_FIRST_AUDIO_STREAM, 0, NULL, NULL, NULL, NULL);
 }
 
-//from IUnknown
-STDMETHODIMP MFDecoderSourceReader::QueryInterface(REFIID riid, LPVOID *ppvObject)
-{
-    if (!ppvObject)
-        return E_POINTER;
-    if (riid == IID_IMFSourceReaderCallback) {
-        *ppvObject = static_cast<IMFSourceReaderCallback*>(this);
-    } else if (riid == IID_IUnknown) {
-        *ppvObject = static_cast<IUnknown*>(this);
-    } else {
-        *ppvObject =  NULL;
-        return E_NOINTERFACE;
-    }
-    AddRef();
-    return S_OK;
-}
-
-STDMETHODIMP_(ULONG) MFDecoderSourceReader::AddRef()
-{
-    return InterlockedIncrement(&m_cRef);
-}
-
-STDMETHODIMP_(ULONG) MFDecoderSourceReader::Release()
-{
-    LONG cRef = InterlockedDecrement(&m_cRef);
-    if (cRef == 0) {
-        this->deleteLater();
-    }
-    return cRef;
-}
-
-//from IMFSourceReaderCallback
-STDMETHODIMP MFDecoderSourceReader::OnReadSample(HRESULT hrStatus, DWORD dwStreamIndex,
-    DWORD dwStreamFlags, LONGLONG llTimestamp, IMFSample *pSample)
-{
-    Q_UNUSED(hrStatus);
-    Q_UNUSED(dwStreamIndex);
-    Q_UNUSED(llTimestamp);
-    if (pSample) {
-        emit newSample(ComPtr<IMFSample>{pSample});
-    } else if ((dwStreamFlags & MF_SOURCE_READERF_ENDOFSTREAM) == MF_SOURCE_READERF_ENDOFSTREAM) {
-        emit finished();
-    }
-    return S_OK;
-}
-
 QT_END_NAMESPACE
 
 #include "moc_mfdecodersourcereader_p.cpp"
